package atomicstryker.minions.common.jobmanager;

import java.util.List;

import atomicstryker.minions.common.entity.EntityMinion;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.util.BlockPos;
import net.minecraftforge.common.ForgeHooks;

/**
 * Blocktask for destroying and stashing a single Block
 * 
 * 
 * @author AtomicStryker
 */

public class BlockTask_MineBlock extends BlockTask
{
    public IBlockState blockState;
    public boolean disableDangerCheck;
	
    /**
     * @param boss Job Manager calling
     * @param input Minion to do it, can remain null while the Job is on wait
     * @param ix coordinate
     * @param iy coordinate
     * @param iz coordinate
     */
    public BlockTask_MineBlock(Minion_Job_Manager boss, EntityMinion input, int ix, int iy, int iz)
    {
    	super(boss, input, ix, iy, iz);
    	disableDangerCheck = false;
    }
    
    /**
     * @param boss Job Manager calling
     * @param input Minion to do it, can remain null while the Job is on wait
     * @param ix coordinate
     * @param iy coordinate
     * @param iz coordinate
     * @param noDangerCheck can disable putting Dirt into air holes and over liquids
     */
    public BlockTask_MineBlock(Minion_Job_Manager boss, EntityMinion input, int ix, int iy, int iz, boolean noDangerCheck)
    {
        this(boss, input, ix, iy, iz);
    	disableDangerCheck = noDangerCheck;
    }

    @Override
    public void onStartedTask()
    {
    	super.onStartedTask();
    }
    
    @Override
    public void onReachedTaskBlock()
    {
    	super.onReachedTaskBlock();
    	
    	blockState = worker.field_70170_p.func_180495_p(pos);
    	//if (blockID > 13) System.out.println("Reached Block["+blockID+"], name "+Block.blocksList[blockID].getBlockName());
    	
    	if (blockState.func_177230_c() == Blocks.field_150350_a)
    	{
    		this.onFinishedTask();
    	}
    }
    
    @Override
    public void onUpdate()
    {
    	super.onUpdate();
    }
    
    @Override
    public void onFinishedTask()
    {
    	super.onFinishedTask();
    	
    	checkDangers();
    	
    	blockState = worker.field_70170_p.func_180495_p(pos); // check against interference mining
    	if (blockState.func_177230_c() != Blocks.field_150350_a && blockState.func_177230_c().func_176195_g(worker.field_70170_p, pos) >= 0F)
    	{
    	    List<ItemStack> stackList = getItemStacksFromWorldBlock(worker.field_70170_p, posX, posY, posZ);
    	    
            int event = ForgeHooks.onBlockBreakEvent(worker.field_70170_p, worker.field_70170_p.func_72912_H().func_76077_q(), 
                    (EntityPlayerMP) worker.master, pos);
            if (event != -1)
            {
                if (worker.field_70170_p.func_175698_g(pos))
                {
                    putBlockHarvestInWorkerInventory(stackList);
                }
            }
    	}
    }
    
    protected void checkDangers()
    {
    	if (!disableDangerCheck)
    	{
	    	// check adjacent blocks for fluids or holes, put safe blocks down
	    	checkBlockForDanger(posX, posY-1, posZ, true);
	    	//checkBlockForDanger(posX, posY+1, posZ);
	    	checkBlockForDanger(posX+1, posY, posZ);
	    	checkBlockForDanger(posX-1, posY, posZ);
	    	checkBlockForDanger(posX, posY, posZ+1);
	    	checkBlockForDanger(posX, posY, posZ-1);
    	}
    	
    	checkBlockForCaveIn(posX, posY+1, posZ);
    }
    
    private void checkBlockForCaveIn(int x, int y, int z)
    {
    	Block checkBlockID = worker.field_70170_p.func_180495_p(new BlockPos(x, y, z)).func_177230_c();
        if (checkBlockID == Blocks.field_150354_m || checkBlockID == Blocks.field_150351_n)
        {
            int event = ForgeHooks.onBlockBreakEvent(worker.field_70170_p, worker.field_70170_p.func_72912_H().func_76077_q(), 
                    (EntityPlayerMP) worker.master, pos);
            if (event != -1)
            {
                putBlockHarvestInWorkerInventory(getItemStacksFromWorldBlock(worker.field_70170_p, posX, posY, posZ));
                
                this.worker.inventory.consumeInventoryItem(Blocks.field_150346_d);
                this.worker.field_70170_p.func_180501_a(pos, Blocks.field_150346_d.func_176223_P(), 3);
            }
        }
	}
    
    private void checkBlockForDanger(int x, int y, int z)
    {
    	this.checkBlockForDanger(x, y, z, false);
    }
    
    private void checkBlockForDanger(int x, int y, int z, boolean putFloor)
    {
    	Block checkBlockID = worker.field_70170_p.func_180495_p(new BlockPos(x, y, z)).func_177230_c();
    	boolean replaceBlock = false;
    	
    	if (checkBlockID == Blocks.field_150350_a)
    	{
    		if (putFloor)
    		{
    			replaceBlock = true;
    		}
    	}
    	else if (!checkBlockID.func_149688_o().func_76220_a() && checkBlockID != Blocks.field_150478_aa)
    	{
    		replaceBlock = true;
    	}
    	
    	if (replaceBlock)
    	{
    	    int event = ForgeHooks.onBlockBreakEvent(worker.field_70170_p, worker.field_70170_p.func_72912_H().func_76077_q(), 
    	            (EntityPlayerMP) worker.master, pos);
            if (event != -1)
            {
                if (checkBlockID != Blocks.field_150350_a)
                {
                    List<ItemStack> stackList = getItemStacksFromWorldBlock(worker.field_70170_p, posX, posY, posZ);
                    if (this.worker.field_70170_p.func_175698_g(pos))
                    {
                        putBlockHarvestInWorkerInventory(stackList);
                    }
                }
                
                this.worker.inventory.consumeInventoryItem(Blocks.field_150346_d);
                this.worker.field_70170_p.func_180501_a(pos, Blocks.field_150346_d.func_176223_P(), 3);
            }
    	}
    }
}
